/*
 * adc.c
 *
 *  Created on: 17.01.2019
 *      Author: AndreR
 */

/**
 * ADC calculations:
t_adc :12.5$
t_smp :28.5 $
f_adc :12e6 $
t_conv :(t_adc+t_smp)/f_adc;
t_total :7*8*t_conv;
mum_samples :333e-6 /t_conv;
 */

#include "adc.h"
#include "usart1.h"
#include "system_init.h"
#include <string.h>

#define NUM_SAMPLES 8
#define NUM_CHANNELS 7
#define NUM_DATAPOINTS (NUM_SAMPLES*NUM_CHANNELS)

#define IR_OUT_BARRIER GPIO_PIN_0
#define IR_OUT_1 GPIO_PIN_5
#define IR_OUT_2 GPIO_PIN_6
#define IR_OUT_3 GPIO_PIN_7
#define IR_OUT_4 GPIO_PIN_3
#define IR_OUT_5 GPIO_PIN_4
#define IR_OUT_6 GPIO_PIN_1
#define IR_OUT_PIN_ALL (GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_3 | GPIO_PIN_4 | GPIO_PIN_5 | GPIO_PIN_6 | GPIO_PIN_7)
#define IR_OUT_PIN_OFF (IR_OUT_PIN_ALL << 16)

static uint16_t dmaData[NUM_DATAPOINTS];
static uint8_t currentPinConfig = 0;
static uint16_t lastMeasurement[3][NUM_CHANNELS];
IrExchangeMISO miso;

static const uint32_t pinConfigs[] = {
	IR_OUT_PIN_OFF,
	IR_OUT_PIN_OFF | IR_OUT_BARRIER,
	IR_OUT_PIN_OFF | IR_OUT_1,

	IR_OUT_PIN_OFF,
	IR_OUT_PIN_OFF | IR_OUT_BARRIER,
	IR_OUT_PIN_OFF | IR_OUT_2,

	IR_OUT_PIN_OFF,
	IR_OUT_PIN_OFF | IR_OUT_BARRIER,
	IR_OUT_PIN_OFF | IR_OUT_3,

	IR_OUT_PIN_OFF,
	IR_OUT_PIN_OFF | IR_OUT_BARRIER,
	IR_OUT_PIN_OFF | IR_OUT_4,

	IR_OUT_PIN_OFF,
	IR_OUT_PIN_OFF | IR_OUT_BARRIER,
	IR_OUT_PIN_OFF | IR_OUT_5,

	IR_OUT_PIN_OFF,
	IR_OUT_PIN_OFF | IR_OUT_BARRIER,
	IR_OUT_PIN_OFF | IR_OUT_6,
};

void DMA1_Channel1_IRQHandler()
{
	// clear flags and prepare DMA and ADC for next trigger from timer
	DMA1_Channel1->CCR &= ~DMA_CCR_EN;
	DMA1->IFCR = DMA_IFCR_CGIF1;
	DMA1_Channel1->CNDTR = NUM_DATAPOINTS;
	DMA1_Channel1->CCR |= DMA_CCR_EN;

	uint8_t pinConfig = currentPinConfig;
	++currentPinConfig;
	currentPinConfig %= sizeof(pinConfigs)/sizeof(pinConfigs[0]);

	// select next output configuration
	GPIOB->BSRR = pinConfigs[currentPinConfig];

	uint8_t slot = pinConfig % 3;

	// process ADC data
	for(uint16_t i = 0; i < NUM_CHANNELS; ++i)
		lastMeasurement[slot][i] = 0;

	for(uint16_t i = 0; i < NUM_DATAPOINTS; ++i)
		lastMeasurement[slot][i%NUM_CHANNELS] += dmaData[i];

	// DMA clears ADSTART bit, set it again to listen for next TIM3 trigger
	ADC1->CR |= ADC_CR_ADSTART;

	if(slot == 2)
	{
		if(lastMeasurement[1][0] > lastMeasurement[0][0])
			miso.channels[0] = lastMeasurement[1][0] - lastMeasurement[0][0];
		else
			miso.channels[0] = 0;

		for(uint16_t i = 1; i < NUM_CHANNELS; ++i)
		{
			if(lastMeasurement[2][i] > lastMeasurement[0][i])
				miso.channels[i] = lastMeasurement[2][i] - lastMeasurement[0][i];
			else
				miso.channels[i] = 0;
		}

		for(uint16_t i = 0; i < 3; ++i)
		{
			for(uint16_t j = 0; j < NUM_CHANNELS; ++j)
				lastMeasurement[i][j] = 0;
		}

		miso.config = pinConfig/3;

		uint8_t* pIrData = (uint8_t*)miso.channels;
		uint8_t xor = 0;
		for(uint16_t i = 0; i < sizeof(miso.channels)+sizeof(miso.config); ++i)
			xor ^= pIrData[i];

		miso.xorChecksum = xor;

		USART1Transmit(&miso, sizeof(miso));
	}
}

void ADCInit()
{
	RCC->APB2ENR |= RCC_APB2ENR_ADCEN;

	// select 12MHz PCLK as source
	ADC1->CFGR2 = ADC_CFGR2_JITOFFDIV4;

	// do ADC calibration
	ADC1->CR = ADC_CR_ADCAL;
	while(ADC1->CR & ADC_CR_ADCAL);

	// configure analog input pins
	GPIOInitData gpioInit;
	gpioInit.mode = GPIO_MODE_ANALOG;
	gpioInit.pupd = GPIO_PUPD_NONE;
	GPIOInit(GPIOA, GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3 | GPIO_PIN_4 | GPIO_PIN_5 | GPIO_PIN_6, &gpioInit);

	GPIOB->BSRR = IR_OUT_PIN_OFF;

	gpioInit.mode = GPIO_MODE_OUTPUT;
	gpioInit.otype = GPIO_OTYPE_PUSH_PULL;
	gpioInit.ospeed = GPIO_OSPEED_2MHZ;
	gpioInit.pupd = GPIO_PUPD_NONE;
	GPIOInit(GPIOB, IR_OUT_PIN_ALL, &gpioInit);

	ADC1->ISR = ADC_ISR_ADRDY;
	ADC1->CR = ADC_CR_ADEN;
	while((ADC1->ISR & ADC_ISR_ADRDY) == 0);

	ADC1->SMPR = ADC_SMPR_SMP_0 | ADC_SMPR_SMP_1; // 28.5 ADC clock cycles sampling time

	ADC1->CHSELR = 0x7F; // channel 0 - 6

	// continouos mode, DMA enabled, external trigger TRG3 (TIM3 TRGO), rising edge
	ADC1->CFGR1 = ADC_CFGR1_CONT | ADC_CFGR1_DMAEN | ADC_CFGR1_EXTSEL_0 | ADC_CFGR1_EXTSEL_1 | ADC_CFGR1_EXTEN_0;

	// ADC, very high priority, memory increment, peripheral to memory, 16bit data size
	DMA1_Channel1->CCR = DMA_CCR_MINC | DMA_CCR_PL | DMA_CCR_MSIZE_0 | DMA_CCR_PSIZE_0;
	DMA1_Channel1->CMAR = (uint32_t)dmaData;
	DMA1_Channel1->CPAR = (uint32_t)&ADC1->DR;
	DMA1_Channel1->CNDTR = NUM_DATAPOINTS;

	DMA1->IFCR = DMA_IFCR_CGIF1;

	DMA1_Channel1->CCR |= DMA_CCR_TCIE | DMA_CCR_TEIE | DMA_CCR_EN;

	NVIC_SetPriority(DMA1_Channel1_IRQn, 5);
	NVIC_EnableIRQ(DMA1_Channel1_IRQn);

	ADC1->CR |= ADC_CR_ADSTART;

	// Setup TIM3 as 333us trigger source
	RCC->APB1ENR |= RCC_APB1ENR_TIM3EN;

	TIM3->CR1 = 0;
	TIM3->CR2 = TIM_CR2_MMS_1; // Update event used as trigger output (TRGO)
	TIM3->SMCR = 0;
	TIM3->CNT = 0;
	TIM3->PSC = 15; // => 3Mhz
	TIM3->ARR = 999; // => 3kHz
	TIM3->CR1 |= TIM_CR1_CEN;
}
