/*
 * main.c
 *
 *  Created on: 14.01.2019
 *      Author: AndreR
 */

/**
 * Pinout:
 *
 *  PA0: IR-ADC0 (Barrier)
 *  PA1: IR-ADC1 (rightmost LED)
 *  PA2: IR-ADC2
 *  PA3: IR-ADC3
 *  PA4: IR-ADC4
 *  PA5: IR-ADC5
 *  PA6: IR-ADC6 (leftmost LED)
 *  PA7:
 *  PA8:
 *  PA9: USART1-TX
 * PA10: USART1-RX
 * PA11:
 * PA12: IR-LED
 * PA13: SWDIO
 * PA14: SWCLK
 * PA15:
 *
 *  PB0: IR0 (Barrier)
 *  PB1: IR6 (leftmost LED)
 *  PB3: IR4
 *  PB4: IR5
 *  PB5: IR1
 *  PB6: IR2
 *  PB7: IR3
 *
 *  PF0: OSC
 *  PF1:
 */

#include "system_init.h"
#include "usart1.h"
#include "adc.h"

#define SYSTEM_CLOCK_HZ 48000000UL
#define SYSTEM_TICK_FREQ 1000

#define TOGGLE_PRIMARY_LED() GPIOA->ODR ^= GPIO_ODR_12

int main()
{
	uint32_t idleCounter;
	uint32_t timeMs = 0;

	// Set AHB and APB1/APB2 clocks to 48MHz
	SystemInit(SYSTEM_CLOCK_HZ, SYSTEM_TICK_FREQ);

	uint32_t idleIncrements = SystemMeasureIdleIncrements();
	if(idleIncrements > UINT16_MAX)
		idleIncrements = UINT16_MAX;

	RCC->AHBENR |= RCC_AHBENR_GPIOAEN | RCC_AHBENR_GPIOBEN | RCC_AHBENR_DMAEN;

	// HAL
	USART1Init();
	ADCInit();

	// main LED
	GPIOA->MODER |= GPIO_MODER_MODER12_0;
	GPIOA->BSRR |= GPIO_BSRR_BR_12;

	while(1)
	{
		idleCounter = SystemWaitUntilSystick();
		if(idleCounter > UINT16_MAX)
			idleCounter = UINT16_MAX;

		miso.idleTicksNoLoad = idleIncrements;
		miso.idleTicksActive = idleCounter;

		++timeMs;

		if(timeMs % 512 == 0)
			TOGGLE_PRIMARY_LED();
	}

	return 0;
}
