/*
 * versions.c
 *
 *  Created on: 01.07.2019
 *      Author: AndreR
 */

#include "versions.h"
#include "main/version.h"
#include "util/boot.h"
#include <stdio.h>

static struct _handles
{
	GHandle hMainVersion;
	GHandle hMainCRC;
	GHandle hMotorVersion;
	GHandle hIrVersion;
	GHandle hExtVersion;
	GHandle hExtBuildDate;
} handles;

static GHandle createLabel(coord_t x, coord_t y, coord_t w, coord_t h, GHandle parent, const char* pText)
{
	GWidgetInit deviceLabelInit = { { x, y, w, h, TRUE, parent }, pText, 0, 0, 0 };
	return gwinLabelCreate(0, &deviceLabelInit);
}

GHandle VersionsCreate()
{
    GWidgetInit wi = { { 0, 60, 240, 260, FALSE, 0 }, "Versions", 0, 0, 0 };
    GHandle hTop = gwinContainerCreate(0, &wi, 0);

    font_t curFont = gwinGetDefaultFont();
    gwinSetDefaultFont(gdispOpenFont("DejaVuSans10"));

    // Main
	static char versionBuf[8];
	uint32_t version = VersionGet();
	snprintf(versionBuf, 8, "%u.%u", version >> 16, version & 0xFFFF);

	static char crcBuf[10];
	uint32_t crc = BootGetApplicationCRC32();
	snprintf(crcBuf, 10, "%08X", crc);

    createLabel(5, 5, 70, 15, hTop, "Main Version:");
    handles.hMainVersion = createLabel(80, 5, 160, 15, hTop, versionBuf);

    createLabel(180, 5, 50, 15, hTop, "Main CRC:");
    handles.hMainCRC = createLabel(210, 5, 60, 15, hTop, crcBuf);

    // Sub-Processors
    createLabel(5, 20, 70, 15, hTop, "Motor Version:");
    handles.hMotorVersion = createLabel(80, 20, 160, 15, hTop, "?");

    createLabel(5, 35, 70, 15, hTop, "IR Version:");
    handles.hIrVersion = createLabel(80, 35, 160, 15, hTop, "?");

    // Ext
    createLabel(5, 50, 70, 15, hTop, "Ext Version:");
    handles.hExtVersion = createLabel(80, 50, 160, 15, hTop, "012345");

    createLabel(5, 65, 70, 15, hTop, "Ext Build Date:");
    handles.hExtBuildDate = createLabel(80, 65, 160, 15, hTop, "42 MB");

    gwinSetDefaultFont(curFont);

    return hTop;
}

void VersionsUpdateExt()
{
	// TODO: fill on branch
}

void VersionsUpdateMotor(uint8_t motorId, STBootloaderFlashResult* pFlashResult)
{
	(void)motorId; // not used yet

	static char buf[8];

	snprintf(buf, 8, "%hu.%hu", pFlashResult->versionMajor, pFlashResult->versionMinor);
	gwinSetText(handles.hMotorVersion, buf, FALSE);
}

void VersionsUpdateIr(STBootloaderFlashResult* pFlashResult)
{
	static char buf[8];

	snprintf(buf, 8, "%hu.%hu", pFlashResult->versionMajor, pFlashResult->versionMinor);
	gwinSetText(handles.hIrVersion, buf, FALSE);
}
