/*
 * led.c
 *
 *  Created on: 30.12.2018
 *      Author: AndreR
 */

#include "led.h"
#include "ch.h"
#include "util/init_hal.h"
#include <math.h>

typedef struct _LEDGlobal
{
	uint16_t state;
} LEDGlobal;

static LEDGlobal led;

void LEDInit()
{
	GPIOInitData gpioInit;
	gpioInit.mode = GPIO_MODE_OUTPUT;
	gpioInit.otype = GPIO_OTYPE_OPEN_DRAIN;
	gpioInit.ospeed = GPIO_OSPEED_2MHZ;
	gpioInit.pupd = GPIO_PUPD_UP;
	GPIOInit(GPIOC, GPIO_PIN_0 | GPIO_PIN_3, &gpioInit); // Mainboard LEDs

	gpioInit.otype = GPIO_OTYPE_PUSH_PULL;
	gpioInit.pupd = GPIO_PUPD_NONE;
	GPIOInit(GPIOB, GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_8 | GPIO_PIN_9, &gpioInit);
	GPIOInit(GPIOC, GPIO_PIN_6 | GPIO_PIN_7, &gpioInit);
	GPIOInit(GPIOD, GPIO_PIN_12 | GPIO_PIN_13, &gpioInit);

//	GPIOSet(GPIOB, GPIO_PIN_0); // left, green
//	GPIOSet(GPIOB, GPIO_PIN_1); // left, red
//	GPIOSet(GPIOB, GPIO_PIN_8); // right, green
//	GPIOSet(GPIOB, GPIO_PIN_9); // right, red
//	GPIOSet(GPIOC, GPIO_PIN_6); // left, white
//	GPIOSet(GPIOC, GPIO_PIN_7); // left, blue
//	GPIOSet(GPIOD, GPIO_PIN_12); // right, white
//	GPIOSet(GPIOD, GPIO_PIN_13); // right, blue

	LEDSet(0);

	RCC->APB1LENR |= RCC_APB1LENR_TIM3EN | RCC_APB1LENR_TIM4EN;
	__DSB();

	TIM3->CCMR1 = TIM_CCMR1_OC1M_1 | TIM_CCMR1_OC1M_2 // PWM mode 1
			| TIM_CCMR1_OC2M_1 | TIM_CCMR1_OC2M_2 // PWM mode 1
			| TIM_CCMR1_OC1PE | TIM_CCMR1_OC2PE; // preload enabled on CCR1 and CCR2
	TIM3->CCMR2 = TIM_CCMR2_OC3M_1 | TIM_CCMR2_OC3M_2 // PWM mode 1
			| TIM_CCMR2_OC4M_1 | TIM_CCMR2_OC4M_2 // PWM mode 1
			| TIM_CCMR2_OC3PE | TIM_CCMR2_OC4PE; // preload enabled on CCR3 and CCR4
	TIM3->CCER = TIM_CCER_CC1E | TIM_CCER_CC2E | TIM_CCER_CC3E | TIM_CCER_CC4E;
	TIM3->PSC = 0; // => 100MHz
	TIM3->ARR = 9999; // => 10kHz
	TIM3->CCR1 = 0;
	TIM3->CCR2 = 0;
	TIM3->CCR3 = 0;
	TIM3->CCR4 = 0;
	TIM3->CR1 = TIM_CR1_CEN;

	TIM4->CCMR1 = TIM_CCMR1_OC1M_1 | TIM_CCMR1_OC1M_2 // PWM mode 1
			| TIM_CCMR1_OC2M_1 | TIM_CCMR1_OC2M_2 // PWM mode 1
			| TIM_CCMR1_OC1PE | TIM_CCMR1_OC2PE; // preload enabled on CCR1 and CCR2
	TIM4->CCMR2 = TIM_CCMR2_OC3M_1 | TIM_CCMR2_OC3M_2 // PWM mode 1
			| TIM_CCMR2_OC4M_1 | TIM_CCMR2_OC4M_2 // PWM mode 1
			| TIM_CCMR2_OC3PE | TIM_CCMR2_OC4PE; // preload enabled on CCR3 and CCR4
	TIM4->CCER = TIM_CCER_CC1E | TIM_CCER_CC2E | TIM_CCER_CC3E | TIM_CCER_CC4E;
	TIM4->PSC = 0; // => 100MHz
	TIM4->ARR = 9999; // => 10kHz
	TIM4->CCR1 = 0;
	TIM4->CCR2 = 0;
	TIM4->CCR3 = 0;
	TIM4->CCR4 = 0;
	TIM4->CR1 = TIM_CR1_CEN;

	gpioInit.mode = GPIO_MODE_AF;
	gpioInit.ospeed = GPIO_OSPEED_2MHZ;
	gpioInit.otype = GPIO_OTYPE_PUSH_PULL;
	gpioInit.pupd = GPIO_PUPD_NONE;
	gpioInit.alternate = 2;

	GPIOInit(GPIOB, GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_8 | GPIO_PIN_9, &gpioInit);
	GPIOInit(GPIOC, GPIO_PIN_6 | GPIO_PIN_7, &gpioInit);
	GPIOInit(GPIOD, GPIO_PIN_12 | GPIO_PIN_13, &gpioInit);

	// LED left
//	TIM3->CCR1 = 100; // white
//	TIM3->CCR2 = 500; // blue
//	TIM3->CCR3 = 500; // green
//	TIM3->CCR4 = 500; // red

	// LED right
//	TIM4->CCR1 = 100; // white
//	TIM4->CCR2 = 500; // blue
//	TIM4->CCR3 = 500; // green
//	TIM4->CCR4 = 500; // red
}

void LEDSet(uint16_t state)
{
	if(state & LED_MAIN_RED)
		GPIOReset(GPIOC, GPIO_PIN_3);
	else
		GPIOSet(GPIOC, GPIO_PIN_3);

	if(state & LED_MAIN_GREEN)
		GPIOReset(GPIOC, GPIO_PIN_0);
	else
		GPIOSet(GPIOC, GPIO_PIN_0);

	uint16_t leftLED = state & LED_FRONT_LEFT_MASK;
	uint16_t rightLED = state & LED_FRONT_RIGHT_MASK;

	if(leftLED == LED_FRONT_LEFT_RED)
		GPIOSet(GPIOB, GPIO_PIN_1);
	else
		GPIOReset(GPIOB, GPIO_PIN_1);

	if(leftLED == LED_FRONT_LEFT_GREEN)
		GPIOSet(GPIOB, GPIO_PIN_0);
	else
		GPIOReset(GPIOB, GPIO_PIN_0);

	if(leftLED == LED_FRONT_LEFT_BLUE)
		GPIOSet(GPIOC, GPIO_PIN_7);
	else
		GPIOReset(GPIOC, GPIO_PIN_7);

	if(leftLED == LED_FRONT_LEFT_WHITE)
		GPIOSet(GPIOC, GPIO_PIN_6);
	else
		GPIOReset(GPIOC, GPIO_PIN_6);

	if(rightLED == LED_FRONT_RIGHT_RED)
		GPIOSet(GPIOB, GPIO_PIN_9);
	else
		GPIOReset(GPIOB, GPIO_PIN_9);

	if(rightLED == LED_FRONT_RIGHT_GREEN)
		GPIOSet(GPIOB, GPIO_PIN_8);
	else
		GPIOReset(GPIOB, GPIO_PIN_8);

	if(rightLED == LED_FRONT_RIGHT_BLUE)
		GPIOSet(GPIOD, GPIO_PIN_13);
	else
		GPIOReset(GPIOD, GPIO_PIN_13);

	if(rightLED == LED_FRONT_RIGHT_WHITE)
		GPIOSet(GPIOD, GPIO_PIN_12);
	else
		GPIOReset(GPIOD, GPIO_PIN_12);

	led.state = state;
}

void LEDMustOn(uint16_t on)
{
	LEDSet(led.state | on);
}

void LEDMustOff(uint16_t off)
{
	LEDSet(led.state & ~(off));
}

void LEDToggle(uint16_t toggle)
{
	LEDSet(led.state ^ toggle);
}

uint16_t LEDGetState()
{
	return led.state;
}

void LEDLeftSet(float red, float green, float blue, float white)
{
	float sum = red + green + blue + white;

	// scale if maximum power exceeds 1W
	if(sum > 1.0f)
	{
		float scale = 1.0f/sum;
		red *= scale;
		green *= scale;
		blue *= scale;
		white *= scale;
	}

	TIM3->CCR1 = (uint16_t)(white*9999.0f);
	TIM3->CCR2 = (uint16_t)(blue*9999.0f);
	TIM3->CCR3 = (uint16_t)(green*9999.0f);
	TIM3->CCR4 = (uint16_t)(red*9999.0f);
}

void LEDRightSet(float red, float green, float blue, float white)
{
	float sum = red + green + blue + white;

	// scale if maximum power exceeds 1W
	if(sum > 1.0f)
	{
		float scale = 1.0f/sum;
		red *= scale;
		green *= scale;
		blue *= scale;
		white *= scale;
	}

	TIM4->CCR1 = (uint16_t)(white*9999.0f);
	TIM4->CCR2 = (uint16_t)(blue*9999.0f);
	TIM4->CCR3 = (uint16_t)(green*9999.0f);
	TIM4->CCR4 = (uint16_t)(red*9999.0f);
}

void LEDDemo()
{
	for(float x = 0.0f; x < 3.14f*10; x += 3.14f/10000.0f)
	{
		LEDLeftSet(fabsf(sinf(x))*0.1f, 0.0f, 0.0f, 0.0f);
		chThdSleepMilliseconds(1);
	}

	LEDSet(LED_FRONT_LEFT_RED | LED_FRONT_RIGHT_RED);
	chThdSleepMilliseconds(500);
	LEDSet(LED_FRONT_LEFT_GREEN | LED_FRONT_RIGHT_GREEN);
	chThdSleepMilliseconds(500);
	LEDSet(LED_FRONT_LEFT_BLUE | LED_FRONT_RIGHT_BLUE);
	chThdSleepMilliseconds(500);
	LEDSet(LED_FRONT_LEFT_WHITE | LED_FRONT_RIGHT_WHITE);
	chThdSleepMilliseconds(500);

	for(uint8_t i = 0; i < 10; i++)
	{
		LEDSet(LED_FRONT_LEFT_RED);
		chThdSleepMilliseconds(100);
		LEDSet(LED_FRONT_RIGHT_RED);
		chThdSleepMilliseconds(100);
	}

	for(uint8_t i = 0; i < 10; i++)
	{
		LEDSet(LED_FRONT_LEFT_GREEN);
		chThdSleepMilliseconds(100);
		LEDSet(LED_FRONT_RIGHT_GREEN);
		chThdSleepMilliseconds(100);
	}

	for(uint8_t i = 0; i < 10; i++)
	{
		LEDSet(LED_FRONT_LEFT_BLUE);
		chThdSleepMilliseconds(100);
		LEDSet(LED_FRONT_RIGHT_BLUE);
		chThdSleepMilliseconds(100);
	}

	for(uint8_t i = 0; i < 10; i++)
	{
		LEDSet(LED_FRONT_LEFT_WHITE);
		chThdSleepMilliseconds(100);
		LEDSet(LED_FRONT_RIGHT_WHITE);
		chThdSleepMilliseconds(100);
	}

	for(uint8_t i = 0; i < 10; i++)
	{
		LEDSet(LED_FRONT_LEFT_RED | LED_FRONT_RIGHT_GREEN);
		chThdSleepMilliseconds(200);
		LEDSet(LED_FRONT_LEFT_GREEN | LED_FRONT_RIGHT_RED);
		chThdSleepMilliseconds(200);
	}

	for(uint8_t i = 0; i < 10; i++)
	{
		LEDSet(LED_FRONT_LEFT_BLUE | LED_FRONT_RIGHT_WHITE);
		chThdSleepMilliseconds(200);
		LEDSet(LED_FRONT_LEFT_WHITE | LED_FRONT_RIGHT_BLUE);
		chThdSleepMilliseconds(200);
	}

	for(uint8_t i = 0; i < 20; i++)
	{
		LEDSet(LED_FRONT_LEFT_RED | LED_FRONT_RIGHT_GREEN);
		chThdSleepMilliseconds(50);
		LEDSet(LED_FRONT_LEFT_GREEN | LED_FRONT_RIGHT_BLUE);
		chThdSleepMilliseconds(50);
		LEDSet(LED_FRONT_LEFT_BLUE | LED_FRONT_RIGHT_WHITE);
		chThdSleepMilliseconds(50);
		LEDSet(LED_FRONT_LEFT_WHITE | LED_FRONT_RIGHT_RED);
		chThdSleepMilliseconds(50);
	}
}
