/*
 * motors.h
 *
 *  Created on: 05.01.2019
 *      Author: AndreR
 */

#pragma once

#include "util/config.h"
#include "util/lag_element.h"
#include "log_msgs.h"

#define MOTORS_DRIBBLER_BAR_RPM_TO_MOTOR_RADS 0.3590391604102621f // rpm *1/60 * 32/12 * 18/14*2 * %pi
#define MOTORS_DRIBBLER_MOTOR_RADS_TO_BAR_RPM 2.785211504108168f

typedef struct _MotorsConfig
{
	float Kp;
	float Ki;
	float Kd;
	float Ktp;
	float Ktd;

	float K;
	float T;
} MotorsConfig;

typedef struct _Motors
{
	uint16_t driveMode;

	struct _drive
	{
		// Control
		PID pid;

		float setVelocity;	// motor rad/s
		float setVoltage;	// set by user, activated by DRIVE_MODE_VOLTAGE
		float setAcceleration;	// motor rad/s^2
		float setTorque; // motor Nm
		float setAngle; // motor electrical angle +-PI
	} drive[4];

	struct _dribbler
	{
		uint8_t mode;

		PID pid;

		LagElementPT1 tempEstimator;
		float temperature; // [deg C]
		uint8_t overheated;

		float setVelocity;	// motor rad/s
		float setVoltage;	// set by user, activated by DRIBBLER_MODE_VOLTAGE
		float setAngle; // motor electrical angle +-PI
	} dribbler;

	MotorsConfig config;
	ConfigFile* pConfigFileMotors;
} Motors;

extern Motors motors;

void MotorsInit();
void MotorsTask(void* params);

void MotorsDriveOff();
void MotorsDriveSetVoltage(uint8_t motor, float vol);
void MotorsDriveSetVelocity(uint8_t motor, float vel);

void MotorsDribblerOff();
void MotorsDribblerSetVoltage(float vol);
void MotorsDribblerSetVelocity(float velRPM);

void MotorsTestSetAngle(uint8_t motor, float angle, float voltage);

