/*
 * motors_com.h
 *
 *  Created on: 05.01.2019
 *      Author: AndreR
 */

#pragma once

#include "util/uart_fifo.h"
#include "util/st_bootloader.h"
#include "log_msgs.h"

#define MOTORS_COM_EVENT_QUEUE_SIZE 8

typedef struct _MotorsSingle
{
	// connection
	UARTFifo uart;
	uint8_t portExTarget;

	// data exchange
	MotorExchangeMISO miso;
	MotorExchangeMOSI mosi;

	// input from motor
	float cpuLoad;
	float avgVoltage1ms;
	float avgCurrent1ms;
	float avgCurrent1ms2;
	float avgTemperature1ms;
	float encoderVelocity; // motor [rad/s]
	float hallVelocity; // motor [rad/s]
	float currentBias;
	uint32_t rxTime;

	// output to motor
	uint8_t mode;
	float setVoltage;	// [V]
	float setCurrent; // [A]
	float setAngle; // [rad]
	uint32_t txTime;

	// config to motor
	float curCtrlKp;
	float curCtrlKi;

	STBootloader bootloader;
	THD_WORKING_AREA(waFlashTask, 256);
	STBootloaderFlashResult flashResult;
	thread_t* pFlashTask;
	uint32_t flashParam;
	int16_t flashFuncResult;
} MotorsSingle;

typedef struct _MotorsCom
{
	uint16_t state;

	uint8_t driveCommutationMode;

	MotorsSingle motor[5];

	mailbox_t eventQueue;
	msg_t pEventQueueData[MOTORS_COM_EVENT_QUEUE_SIZE];

	uint8_t recordMotor;
	uint8_t commKillMotor;
} MotorsCom;

extern MotorsCom motorsCom;

void MotorsComInit();
void MotorsComUpdateFirmware(uint8_t force);

// these 4 functions must be called in-order at 1kHz
void MotorsComRunStatemachine();
void MotorsComReceiveData();
void MotorsComStartSendData();
void MotorsComWaitSendDataComplete();

void MotorsComSetOff(uint8_t motor);
void MotorsComSetVoltage(uint8_t motor, float voltage);
void MotorsComSetCurrent(uint8_t motor, float current);
void MotorsComSetPIGainsCurrent(uint8_t motor, float kp, float ki);

void MotorsComTestSetElectricalAngle(uint8_t motor, float angle, float voltage);

void MotorsComDebugRecord(uint8_t id);
void MotorsComDebugPrintRecording();
