/*
 * prog_store_impl.c
 *
 *  Created on: 23.06.2019
 *      Author: AndreR
 */

#include "prog_store_impl.h"
#include "util/prog_store.h"
#include "util/crc.h"
#include "util/flash.h"
#include "intercom_constants.h"
#include "string.h"

#define PROG_SIZE_MAIN (512*1024)
#define PROG_SIZE_IR (32*1024)
#define PROG_SIZE_MOT (32*1024)

#define PROG_INFO_ADDR	0x08040000
#define PROG_MOT_ADDR	0x08060000
#define PROG_IR_ADDR	0x08070000
#define PROG_MAIN_ADDR	0x08080000

const uint8_t* pProgs[] = {
	(const uint8_t*)PROG_MAIN_ADDR,
	(const uint8_t*)PROG_MOT_ADDR,
	(const uint8_t*)PROG_IR_ADDR,
};

typedef struct PACKED _ProgUpgradePadded
{
	ProgUpgradeInfo info;
	uint8_t pad[32-sizeof(ProgUpgradeInfo)];
} ProgUpgradePadded;

const ProgUpgradePadded* pInfoFlash = (ProgUpgradePadded*)PROG_INFO_ADDR;

void ProgStoreImplInit()
{
	ProgStoreAddTarget(1, PROG_SIZE_MAIN, PROG_STORE_ID_MAIN2019);
	ProgStoreAddTarget(1, PROG_SIZE_MOT, PROG_STORE_ID_MOT2019);
	ProgStoreAddTarget(1, PROG_SIZE_IR, PROG_STORE_ID_IR2019);
}

void ProgStoreImplWriteData(uint8_t procId, uint32_t offset, uint8_t* pData, uint32_t size)
{
	FlashProgram((uint32_t)(pProgs[procId]+offset), (uint32_t*)pData, (size+3)/4);
}

void ProgStoreImplGetUpgradeInfo(uint8_t procId, ProgUpgradeInfo* pInfo)
{
	memcpy(pInfo, &pInfoFlash[procId].info, sizeof(ProgUpgradeInfo));
}

void ProgStoreImplSetUpgradeInfo(uint8_t procId, const ProgUpgradeInfo* pInfo)
{
	ProgUpgradePadded data;
	memset(&data, 0, sizeof(ProgUpgradePadded));
	memcpy(&data.info, pInfo, sizeof(ProgUpgradeInfo));
	FlashProgram((uint32_t)(&pInfoFlash[procId]), (uint32_t*)&data, sizeof(ProgUpgradePadded)/4);
}

void ProgStoreImplReadData(uint8_t procId, uint32_t offset, uint8_t* pBuf, uint32_t size)
{
	memcpy(pBuf, pProgs[procId]+offset, size);
}

uint32_t ProgStoreImplGetChecksum(uint8_t procId, uint32_t offset, uint32_t size)
{
	return CRC32CalcChecksumAlt(pProgs[procId]+offset, size);
}

void ProgStoreImplEraseStorage()
{
	// erase program update info, motor, ir, and main code
	FlashErase(0x08040000, 0x08100000);
}
