/*
 * data.h
 *
 *  Created on: 12.03.2019
 *      Author: AndreR
 */

#pragma once

#include "log_msgs.h"
#include "pi_ctrl_s12.h"

typedef struct _Data
{
	uint32_t timeMs;

	struct _performance
	{
		uint32_t idleIncrements;
		uint32_t idleCounter;
	} performance;

	struct _sensors
	{
		struct _encoder
		{
			uint8_t installed;
			uint16_t position;
			int16_t delta; // within 1ms
			int16_t deltaSaturated; // saturated to -4096-4095
		} encoder;

		struct _hall
		{
			uint8_t position;
			uint32_t commutationTime; // in timer clock cycles
			int32_t speed; // speed = rpm*pp*1/60*1.0/48000000*2^32*1/2^7; (pp = 6 | 48)
			int16_t speedSaturated;
		} hall;

		struct _adc
		{
			int32_t vSupply_Q16_0; // [mV]
			int32_t vSupplyReciprocal_Q0_31;
			int32_t iMotor_Q16_0; // [mA]
		} adc;
	} sensors;

	MotorExchangeMOSI command;

	struct _comm
	{
		uint32_t lastCommandTimeMs;
		uint8_t timeout;
	} comm;

	struct _motor
	{
		uint8_t mode;
		uint16_t angle;
		int32_t voltage;

		volatile uint8_t breakActive;
	} motor;

	struct _ctrl
	{
		PICtrlS12 current;

		int32_t iMotorBias; // [mA]
	} ctrl;
} Data;

extern Data data;
