/*
 * enc.c
 *
 *  Created on: 21.01.2019
 *      Author: AndreR
 */

#include "enc.h"
#include "system_init.h"
#include "data.h"
#include "motor.h"

#define GPIO_PIN_ENCODER (GPIO_PIN_6 | GPIO_PIN_7)

void EncInit()
{
	GPIOInitData gpioInit;
	gpioInit.mode = GPIO_MODE_INPUT;
	gpioInit.pupd = GPIO_PUPD_DOWN;
	GPIOInit(GPIOA, GPIO_PIN_ENCODER, &gpioInit);

	SystemWaitMs(10);

	uint16_t pullDownState = GPIOA->IDR & GPIO_PIN_ENCODER;

	gpioInit.pupd = GPIO_PUPD_UP;
	GPIOInit(GPIOA, GPIO_PIN_ENCODER, &gpioInit);

	SystemWaitMs(10);

	uint16_t pullUpState = GPIOA->IDR & GPIO_PIN_ENCODER;

	if(pullDownState == 0 && pullUpState == GPIO_PIN_ENCODER)
		data.sensors.encoder.installed = 0;
	else
		data.sensors.encoder.installed = 1;

	gpioInit.mode = GPIO_MODE_AF;
	gpioInit.alternate = 1;
	gpioInit.pupd = GPIO_PUPD_NONE;
	GPIOInit(GPIOA, GPIO_PIN_ENCODER, &gpioInit);

	RCC->APB1ENR |= RCC_APB1ENR_TIM3EN;

	// Encoder max 36864cpr, 6080 RPM, 0.13us edge separation => 7.7MHz
	TIM3->CR1 = 0;
	TIM3->CR2 = 0;
	TIM3->PSC = 0;
	TIM3->ARR = 36863;
	TIM3->DIER = 0;
	// encoder mode 3, count all flanks, Internal Trigger 1 (TIM2) on TRC
	TIM3->SMCR = (3 << TIM_SMCR_SMS_Pos);
	// IC1 mapped to TI1, IC2 mapped to TI2, filter = 0x02 on both (4 samples at 48MHz), no prescaler
	TIM3->CCMR1 = TIM_CCMR1_CC1S_0 | (2 << TIM_CCMR1_IC1F_Pos) | TIM_CCMR1_CC2S_0 | (2 << TIM_CCMR1_IC2F_Pos);
	TIM3->CCER = TIM_CCER_CC1E | TIM_CCER_CC2E;	// enable input capture, rising polarity
	TIM3->CR1 = TIM_CR1_CEN;
}

void EncUpdate()
{
	static uint16_t lastEncPos = 0;

	uint16_t encPos = motor.encDmaData[0];

	int32_t encDelta = (int32_t)encPos - (int32_t)lastEncPos;
	if(encDelta > 18432)
		encDelta -= 36864;
	if(encDelta < -18432)
		encDelta += 36864;

	lastEncPos = encPos;

	// bring encoder delta in hall delta range, 8 pole pairs => hallSpeed = encDelta * 0.9102
	encDelta = (encDelta*59652) >> 16;

	data.sensors.encoder.delta = encDelta;
	data.sensors.encoder.position = encPos;

	if(encDelta > 4095)
		encDelta = 4095;
	if(encDelta < -4096)
		encDelta = -4096;

	data.sensors.encoder.deltaSaturated = encDelta;
}
