/*
 * pid_s12.c
 *
 *  Created on: 06.03.2019
 *      Author: AndreR
 *
 *  Fixed-Point PI Controller
 *  Input/Output: S12.0
 */

#include "pi_ctrl_s12.h"

/**
 * measured: S12.0
 */
void PICtrlS12Update(PICtrlS12* pPI, int32_t measured)
{
	if(!pPI->enabled)
		return;

	if(measured > 4095)
		measured = 4095;
	if(measured < -4096)
		measured = -4096;

	// S13.0 = S12.0 - S12.0
	int32_t error = pPI->setpoint - measured;

	// S13.18 = S0.18 * S13.0
	int32_t iError = pPI->ki * error;

	// => S13.17
	iError >>= 1;

	// S14.17 = S12.17 + S13.17
	int32_t iTermNew = pPI->iTerm + iError;

	// => S12.17
	if(iTermNew > (pPI->outputMax << 17))
		iTermNew = pPI->outputMax << 17;
	if(iTermNew < (pPI->outputMin << 17))
		iTermNew = (pPI->outputMin << 17);

	pPI->iTerm = iTermNew;

	// S20.10 = S7.10 * S13.0
	int32_t pError = pPI->kp * error;

	// S21.10 = S20.10 + (S12.17 >> 7)
	int32_t output = pError + (iTermNew >> 7);

	// => S21.0
	output >>= 10;

	// => S12.0
	if(output > pPI->outputMax)
	{
		output = pPI->outputMax;
		pPI->overload = 1;
	}
	else if(output < pPI->outputMin)
	{
		output = pPI->outputMin;
		pPI->overload = 1;
	}
	else
		pPI->overload = 0;

	pPI->output = output;
}

void PICtrlS12Enable(PICtrlS12* pPI, uint8_t enable)
{
	if(enable && !pPI->enabled)
	{
		// we just went from manual to auto
		pPI->iTerm = pPI->output << 17;
	}

	pPI->enabled = enable;
}

void PICtrlS12Init(PICtrlS12* pPI, int32_t kp, int32_t ki, int32_t outputMin, int32_t outputMax)
{
	pPI->kp = kp;
	pPI->ki = ki;
	pPI->outputMin = outputMin;
	pPI->outputMax = outputMax;
	pPI->iTerm = 0;
	pPI->output = 0;
	pPI->overload = 0;
	pPI->setpoint = 0;
	pPI->enabled = 0;
}
